import re
import os
import datetime
import string
from difflib import SequenceMatcher

from cached_property import cached_property

from oireachtas_data.models.speech import Speech
from oireachtas_data.models.para import Para


class Section:
    def __init__(self, data=None, title=None, next_title=None):
        self.data = data
        self.title = title
        self.next_title = next_title

    @property
    def content(self):
        from oireachtas_data.utils import first_occuring

        lines = self.non_header_content.split("\n")

        valid_lines = []
        for line in lines:
            line = line.strip()
            good = True

            try:
                int(line[0])
                datetime.datetime.strptime(line.strip()[:10], "%d/%m/%Y")
                good = False
            except (ValueError, IndexError):
                pass

            if good:
                valid_lines.append(line)

        content = "\n".join(valid_lines)

        data = ""

        if self.next_title is None:
            if self.title in content:
                data = content[content.index(self.title) + len(self.title) + 1 :]
            elif self.title.rstrip(string.digits) in content:
                data = content[
                    content.index(self.title.rstrip(string.digits))
                    + len(self.title.rstrip(string.digits))
                    + 1 :
                ]
            elif len(self.title) > 50 and self.title[:40] in content:
                data = content[content.index(self.title[:40]) + len(self.title[:40]) :]

            else:
                # This could be that the title down below is so long that it's split
                print("oops, could not find the last section")

        else:
            if self.title not in content or self.next_title not in content:
                lines = content.split("\n")

                start_index = None
                end_index = None

                if self.title not in content:
                    modified_title = self.title.rstrip(string.digits)
                    if modified_title in content:
                        start_index = content.index(modified_title)
                    else:
                        from oireachtas_data.utils import window

                        for p, n in window(lines, window_size=2):
                            if p + " " + n == modified_title:
                                start_index = content.index(p + "\n" + n)
                else:
                    start_index = content.index(self.title)

                if start_index is None:
                    # try remove numbers from the end of the title
                    print(f"Could not get {self.title} from content")
                    return ""

                if self.next_title not in content:
                    from oireachtas_data.utils import window

                    for p, n in window(lines, window_size=2):
                        if p + " " + n == self.next_title:
                            end_index = content.index(p + "\n" + n)
                else:
                    end_index = content.index(self.next_title)

                data = content[start_index + len(self.title) + 1 : end_index]
            else:
                data = content[
                    content.index(self.title)
                    + len(self.title)
                    + 1 : content.index(self.next_title)
                ]

        # Remove tables of names from voting
        while (
            "The Dáil divided: Tá," in data
            or "The Committee divided: Tá," in data
            or "The Seanad divided: Tá," in data
        ):
            earliest = first_occuring(
                [
                    "The Dáil divided: Tá,",
                    "The Committee divided: Tá,",
                    "The Seanad divided: Tá,",
                ],
                data,
            )
            first_occuring_string = earliest[1]

            table_start = data.index(first_occuring_string)

            earliest = first_occuring(
                ["Question declared carried.", "Amendment declared lost."],
                data[table_start:],
            )
            if earliest[1] is None:
                break

            table_end = earliest[0]

            start = data[:table_start]
            end = data[table_start + table_end + 5 :]

            data = start + "\n\n" + end

        data = " ".join(data.split("[PAGEBREAK]"))

        return data.strip()

    def __del__(self):
        attrs_to_del = ["non_header_content"]

        for attr in attrs_to_del:
            try:
                delattr(self, attr)
            except AttributeError:
                pass

    @cached_property
    def non_header_content(self):
        return self.data[self.data.rindex("����������������������������������") :]

    @property
    def speeches(self):
        if ":" not in self.content:
            return []

        data = self.content

        lines = data.split("\n")
        new_lines = []
        for line in lines:
            line = line.strip()

            if ":" in line and any(
                [
                    "Deputy" in line,
                    line[: line.index(":")].strip().split(" ")[-1][0].isupper(),
                ]
            ):
                new_lines.append(line)
            else:
                if len(new_lines):
                    new_lines[-1] = new_lines[-1] + " " + line

        speeches = []
        for line in new_lines:
            person = line[0 : line.index(":")].strip()
            content = line[line.index(":") + 1 :]
            content = content.replace("   Seanad Éireann  ", " ").strip()

            speeches.append(
                Speech(
                    by=person,
                    _as=None,
                    eid=None,
                    paras=[Para(title=None, eid=None, content=content)],
                )
            )

        return speeches


class PDF:
    def __init__(self, fp):
        self.fp = fp
        self.data = None
        self.loaded = False

    def load(self):
        text_file = self.fp + ".txt"

        if not os.path.exists(text_file):
            with open(self.fp, "rb") as f:
                os.system(f"pdftotext '{self.fp}' '{text_file}'")

        if not os.path.exists(text_file):
            print("Could not convert pdf to text: %s" % (text_file))
            return

        f = open(text_file, "r")
        self.data = f.read()
        f.close()

        # Just weird thigs to remove, probably wobblyness in conversion to text
        self.data = self.data.replace(
            "-----", " — "
        )  # interruption / continuation of interruption
        self.data = self.data.replace("\x0c", "")

        # Removes timestamps of speach
        self.data = re.sub("\\d+\\/\\d+\\/\\d+[A-Za-z]+\\d+", "", self.data)

        self.data = self.data.replace(
            "An Leas-Chathaoirleach:", "\nAn Leas-Chathaoirleach:"
        )
        self.data = self.data.replace(
            "----An Leas-Chathaoirleach:", "\nAn Leas-Chathaoirleach:"
        )
        self.data = self.data.replace("----An Cathaoirleach:", "\nAn Cathaoirleach:")
        self.data = self.data.replace(
            "----An Ceann Comhairle:", "\nAn Ceann Comhairle:"
        )
        self.data = self.data.replace("----Deputy", "\nDeputy")
        self.data = self.data.replace("----Senator", "\nSenator")
        self.data = self.data.replace("----Acting Chairman", "\nActing Chairman")

        # would be handy to replace all "An Leas-Chathaoirleach:" with "\nAn Leas-Chathaoirleach:" so we can parse things easier really, sometimes they get merged into other words (and other common names)

        lines = self.data.split("\n")
        good_lines = []

        for line in lines:
            line = line.strip()
            good = True
            try:
                int(line[0])
                datetime.datetime.strptime(line.strip(), "%d %B %Y")
                good = False
            except:
                pass

            if line.strip() == "Dáil Éireann":
                good = False

            if good:
                good_lines.append(line)

        lines = good_lines
        good_lines = []

        for line in lines:
            line = line.strip()
            try:
                int(line)
                good_lines.append("[PAGEBREAK]")
            except (ValueError, IndexError):
                good_lines.append(line)

        self.data = "\n".join(good_lines)

    def matching_header(self, show_as):
        for header in self.section_headers:
            if SequenceMatcher(None, header, show_as).ratio() > 0.8:
                return header
        return None

    @property
    def section_headers(self):
        if not self.loaded:
            self.load()

        if self.data is None:
            return []

        lines = self.data.split("\n")
        start = -1
        end = -1
        for idx, line in enumerate(lines):
            # If there's a continue then it may be skipped. Long lines
            # get first and last of ����� then if no ����� in line merge with the next line
            if "��������������" in line and start != -1:
                end = idx
            if "��������������" in line and start == -1:
                start = idx

        cleaned_header_lines = []
        append_next = False
        if end == -1 and start != -1:
            # if end is -1 and start is not, then there's only one thing on the list
            header_lines = [lines[start]]
        else:
            # if there's content on the line before the start idx then it may be a run-on title
            if lines[start - 1].strip():
                start = start - 1
            header_lines = lines[start : end + 1]

        for header_line in header_lines:
            if "\x08" in header_line:
                if append_next:
                    cleaned_header_lines[-1] += (
                        " " + header_line[: header_line.index("�")]
                    )
                else:
                    if "�" not in header_line:
                        # page number at end
                        cleaned_header_lines.append(header_line)
                    else:
                        cleaned_header_lines.append(
                            header_line[: header_line.index("�")]
                        )
                append_next = False
            else:
                cleaned_header_lines.append(header_line)
                append_next = True

        clean_headers = []
        for header in cleaned_header_lines:
            header = header.replace("\x08", "")
            if len(header.split()):
                if header.split()[-1].isnumeric():
                    header = " ".join(header.split()[:-1])

            header = header.strip()
            if "\x08" in header:
                header = header[: header.index("\x08")].replace("\n", " ")

            header = " ".join(header.strip().split("\n")).strip()

            if header:
                try:
                    int(header[0])
                    datetime.datetime.strptime(header, "%d/%m/%YA%M%S0")
                except (ValueError, IndexError):
                    clean_headers.append(header)
        return clean_headers

    @property
    def date(self):
        return datetime.datetime.strptime(
            os.path.splitext(self.fp.split("_")[-1])[0], "%Y-%m-%d"
        )

    @property
    def debate_sections(self):
        if not self.loaded:
            self.load()
        from oireachtas_data.utils import window

        sections = []
        for title, next_title in window(self.section_headers + [None], window_size=2):
            sections.append(Section(data=self.data, title=title, next_title=next_title))
        return sections



import os
import re
from functools import lru_cache

import ujson
import edlib
import unidecode

from oireachtas_data.constants import MEMBERS_DIR


class Members:
    def __init__(self):
        self.loaded = False
        self.data = []
        self.found_member_pids = set()

    def append(self, member):
        self.data.append(member)

    def serialize(self):
        return [m.serialize() for m in self.data]

    def write(self):
        with open(os.path.join(MEMBERS_DIR, "members.json"), "w") as f:
            f.write(ujson.dumps(self.serialize()))

    def load(self):
        if self.loaded:
            return

        if os.path.exists(os.path.join(MEMBERS_DIR, "members.json")):
            with open(os.path.join(MEMBERS_DIR, "members.json"), "r") as f:
                data = ujson.loads(f.read())
                for d in data:
                    self.data.append(Member(**d))
        self.loaded = True

    @lru_cache(maxsize=10000)
    def get_member(self, member_str):
        if member_str.startswith("#"):
            return self.get_member_from_id(member_str)
        else:
            return self.get_member_from_name(member_str)

    @staticmethod
    def clean_name(name):
        # If a minister is specified it usually looks like:
        # Minister for Something (Deputy Joan Malone)
        if "Minister" in name:
            try:
                name = re.findall("\((.*?)\)", name)[0]
            except:
                pass

        name = name.replace(")", "")
        name = name.replace("(", "")
        name = name.replace("Deputy ", "")
        name = name.replace("Senator ", "")
        name = name.replace("Mr. ", "")
        name = name.replace("Ms. ", "")
        name = name.replace("Mrs. ", "")
        name = name.replace(" ", "")
        name = name.replace(".", "")
        name = name.replace("Acting Chairman", "")

        # Remove fadas
        name = unidecode.unidecode(name)

        name = name.replace("'", "")

        return name.strip()

    @lru_cache(maxsize=100)
    def get_probable_members(self, starting_char):
        probable_members = [
            m for m in self.data if m.pid is not None and m.pid[0] == starting_char
        ]
        probable_members_pids = set([m.pid for m in probable_members])
        return probable_members, probable_members_pids

    @lru_cache(maxsize=10000)
    def get_member_from_name(self, name):
        # Some issues with michael collins since there are two
        # FIXME: Any near duplicate names should be noted.
        # if there's a number at the end we need to be very careful

        name = self.clean_name(name)

        if name == "":
            return

        for member in self.data:
            if member.pid is None:
                continue
            if member.pid == name:
                self.found_member_pids.add(member.pid)
                return member
            if member.pid == f"{name}FF":
                self.found_member_pids.add(member.pid)
                return member
            if member.pid == f"{name}LAB":
                self.found_member_pids.add(member.pid)
                return member
            if member.pid == f"{name}FG":
                self.found_member_pids.add(member.pid)
                return member
            if member.pid == f"{name}SF":
                self.found_member_pids.add(member.pid)
                return member
            if member.pid == f"Dr{name}":
                self.found_member_pids.add(member.pid)
                return member

        probable_members, probable_members_pids = self.get_probable_members(name[0])

        for member in probable_members:
            if member.pid is None:
                continue
            if member.pid in self.found_member_pids:
                continue
            if len(set(member.pid).symmetric_difference(name)) > 2:
                continue
            if edlib.align(member.pid, name)["editDistance"] < 4:
                self.found_member_pids.add(member.pid)
                return member

        for member in self.data:
            if member.pid is None:
                continue
            if member.pid in probable_members_pids:
                continue
            if member.pid in self.found_member_pids:
                continue
            if len(set(member.pid).symmetric_difference(name)) > 2:
                continue
            if edlib.align(member.pid, name)["editDistance"] < 4:
                self.found_member_pids.add(member.pid)
                return member

    @lru_cache(maxsize=10000)
    def get_member_from_id(self, pid):
        pid = pid.replace("#", "")

        for member in self.data:
            if member.pid == pid:
                return member

    @lru_cache(maxsize=10000)
    def parties_of_member(self, member):
        if isinstance(member, str):
            member = self.get_member_from_name(member)
            if member is None:
                return None

        parties = []
        for membership in member.memberships:
            parties.append(membership.parties[0].party_code)

        return parties


class Member:
    __slots__ = (
        "date_of_death",
        "first_name",
        "last_name",
        "full_name",
        "gender",
        "member_code",
        "_pid",
        "memberships",
        "_set_pid",
    )

    def __init__(self, *args, **kwargs):
        self.date_of_death = kwargs.get(
            "dateOfDeath", kwargs.get("date_of_death", None)
        )
        self.first_name = kwargs.get("firstName", kwargs.get("first_name", None))
        self.last_name = kwargs.get("lastName", kwargs.get("last_name", None))
        self.full_name = kwargs.get("fullName", kwargs.get("full_name", None))
        self.gender = kwargs.get("gender", kwargs.get("gender", None))
        self.member_code = kwargs.get("memberCode", kwargs.get("member_code", None))
        self._pid = kwargs.get("pId", kwargs.get("pid", None))
        self._set_pid = None

        self.memberships = []
        for membership in kwargs["memberships"]:
            if "membership" in membership:
                self.memberships.append(Membership(**membership["membership"]))
            else:
                self.memberships.append(Membership(**membership))

    @property
    def pid(self):
        if self._set_pid:
            return self._set_pid

        if isinstance(self._pid, str) and self._pid[0] == "#":
            self._set_pid = self._pid.replace("#", "")
            return self._set_pid

        self._set_pid = self._pid
        return self._set_pid

    def serialize(self):
        return {
            "date_of_death": self.date_of_death,
            "first_name": self.first_name,
            "last_name": self.last_name,
            "full_name": self.full_name,
            "gender": self.gender,
            "member_code": self.member_code,
            "pid": self.pid,
            "memberships": [m.serialize() for m in self.memberships],
        }


class House:
    __slots__ = ("chamber_type", "house_code", "house_no", "show_as")

    def __init__(self, *args, **kwargs):
        self.chamber_type = kwargs.get("chamberType", kwargs.get("chamber_type", None))
        self.house_code = kwargs.get("houseCode", kwargs.get("house_code", None))
        self.house_no = kwargs.get("houseNo", kwargs.get("house_no", None))
        self.show_as = kwargs.get("showAs", kwargs.get("show_as", None))

    def serialize(self):
        return {
            "chamber_type": self.chamber_type,
            "house_code": self.house_code,
            "house_no": self.house_no,
            "show_as": self.show_as,
        }


class Party:
    __slots__ = ("date_start", "date_end", "party_code", "show_as")

    def __init__(self, *args, **kwargs):
        if "party" in kwargs:
            base = kwargs["party"]

            date_range = base["dateRange"]

            self.date_start = date_range["start"]
            self.date_end = date_range["end"]

            self.party_code = base["partyCode"]
            self.show_as = base["showAs"]

        else:
            self.date_start = kwargs["date_start"]
            self.date_end = kwargs["date_end"]

            self.party_code = kwargs["party_code"]
            self.show_as = kwargs["show_as"]

    def serialize(self):
        return {
            "date_start": self.date_start,
            "date_end": self.date_end,
            "party_code": self.party_code,
            "show_as": self.show_as,
        }


class Represents:
    __slots__ = ("represent_code", "represent_type", "show_as")

    def __init__(self, *args, **kwargs):
        if "represent" in kwargs:
            base = kwargs["represent"]

            self.represent_code = base["representCode"]
            self.represent_type = base["representType"]
            self.show_as = base["showAs"]

        else:
            self.represent_code = kwargs["represent_code"]
            self.represent_type = kwargs["represent_type"]
            self.show_as = kwargs["show_as"]

    def serialize(self):
        return {
            "represent_code": self.represent_code,
            "represent_type": self.represent_type,
            "show_as": self.show_as,
        }


class Membership:
    __slots__ = ("date_start", "date_end", "house", "offices", "parties", "represents")

    def __init__(self, *args, **kwargs):
        if "dateRange" in kwargs:
            date_range = kwargs["dateRange"]
            self.date_start = date_range.get("start", None)
            self.date_end = date_range.get("end", None)
        else:
            self.date_start = kwargs["date_start"]
            self.date_end = kwargs["date_end"]

        self.house = House(**kwargs["house"])

        self.offices = []
        for office in kwargs["offices"]:
            pass  # TODO

        self.parties = []
        for party in kwargs["parties"]:
            self.parties.append(Party(**party))

        self.represents = []
        for rep in kwargs["represents"]:
            self.represents.append(Represents(**rep))

    def serialize(self):
        return {
            "date_start": self.date_start,
            "date_end": self.date_end,
            "house": self.house.serialize(),
            "offices": [o.serialize() for o in self.offices],
            "parties": [p.serialize() for p in self.parties],
            "represents": [r.serialize() for r in self.represents],
        }



from unittest import TestCase, skip

from oireachtas_data.utils import (
    merge_paras,
    get_file_content,
    get_debates,
    window,
    first_occuring,
    find_nth,
)


class UtilsTest(TestCase):
    def test_find_nth(self):
        self.assertEqual(find_nth("12321", "2", 0), 1)
        self.assertEqual(find_nth("12321", "2", 2), 3)

    @skip("TODO")
    def test_merge_paras(self):
        pass

    @skip("TODO")
    def test_get_file_content(self):
        pass

    @skip("TODO")
    def test_get_debates(self):
        pass

    def test_window(self):
        self.assertEqual(
            list(window(range(10), window_size=2)),
            [(0, 1), (1, 2), (2, 3), (3, 4), (4, 5), (5, 6), (6, 7), (7, 8), (8, 9)],
        )

    def test_first_occuring(self):
        self.assertEqual(
            first_occuring(["three", "two"], "one two three four"), (4, "two")
        )



import os
import datetime
import requests
from collections import defaultdict

import ujson

from oireachtas_data.constants import DEBATES_DIR
from oireachtas_data.models.debate_section import DebateSection


class Debate:
    __slots__ = (
        "date",
        "chamber",
        "counts",
        "debate_sections",
        "debate_type",
        "data_uri",
        "loaded",
        "_json_location",
    )

    def __init__(
        self,
        date=None,
        chamber=None,
        counts=None,
        debate_sections=None,
        debate_type=None,
        data_uri=None,
        loaded=False,
    ):
        """

        :kwarg date: The date of the debate in YYYY-MM-DD format
        :kwarg chamber: Dail or Seanad
        :kwarg counts:
        :kwarg debate_sections: list of debate sections in json from the api
        :kwarg debate_type: the tye of the debate, usually just 'debate'
        :kwarg data_uri:
        """
        self.date = date
        self.chamber = chamber
        self.counts = counts
        self.debate_sections = debate_sections
        self.debate_type = debate_type
        self.data_uri = data_uri

        self.loaded = loaded

        self._json_location = None

    def set_filepath(self, filepath):
        self._json_location = filepath

    @staticmethod
    def from_file(filepath):
        from oireachtas_data.utils import get_file_content

        debate = Debate()
        debate.set_filepath(filepath)
        return debate

    @staticmethod
    def parse(data):
        return Debate(
            date=data["date"],
            chamber=data["chamber"],
            counts=data["counts"],
            debate_sections=[DebateSection.parse(d) for d in data["sections"]],
            debate_type=data["debate_type"],
            data_uri=data["data_uri"],
            loaded=True,
        )

    def load_data(self):
        """
        Load data from the data_uri to populate the debate sections
        """
        if self.loaded:
            return

        if os.path.exists(self.json_location):
            from oireachtas_data.utils import get_file_content

            data = get_file_content(self.json_location)
            self.date = data["date"]
            self.chamber = data["chamber"]
            self.counts = data["counts"]
            self.debate_sections = [DebateSection.parse(d) for d in data["sections"]]
            self.debate_type = data["debate_type"]
            self.data_uri = data["data_uri"]
        else:
            chambers = {"Dáil Éireann": "dail", "Seanad Éireann": "seanad"}
            url = (
                "https://data.oireachtas.ie/ie/oireachtas/debateRecord/%s/%s/debate/mul@/main.pdf"
                % (chambers[self.chamber], self.date)
            )

            pdf_request = requests.get(url, stream=True)

            # not self.pdf_location as recursive error
            pdf_location = os.path.join(
                DEBATES_DIR, "%s_%s_%s.pdf" % ("debate", self.chamber, self.date)
            )

            with open(pdf_location, "wb") as pdfile:
                for chunk in pdf_request.iter_content(2000):
                    pdfile.write(chunk)

            # If we're missing data from the api cause of being forbidden we can get some data from the pdf

            debate_sections = []
            for idx, section in enumerate(self.debate_sections):
                raw_debate_section = section["debateSection"]
                debate_section = DebateSection(
                    bill=raw_debate_section["bill"],
                    contains_debate=raw_debate_section["containsDebate"],
                    counts=raw_debate_section["counts"],
                    debate_section_id=raw_debate_section["debateSectionId"],
                    debate_type=raw_debate_section["debateType"],
                    data_uri=raw_debate_section["formats"]["xml"]["uri"],
                    parent_debate_section=raw_debate_section["parentDebateSection"],
                    show_as=raw_debate_section["showAs"],
                    show_as_idx=idx,
                    show_as_context=[
                        d["debateSection"]["showAs"] for d in self.debate_sections
                    ],
                    speakers=raw_debate_section["speakers"],
                    pdf_location=pdf_location,
                )
                debate_section.load_data()
                debate_sections.append(debate_section)

            self.debate_sections = debate_sections

        self.loaded = True

    def serialize(self):
        self.load_data()
        return {
            "date": self.date,
            "chamber": self.chamber,
            "counts": self.counts,
            "debate_type": self.debate_type,
            "data_uri": self.data_uri,
            "sections": [s.serialize() for s in self.debate_sections],
        }

    def write(self, location=None):
        if location is None:
            location = self.json_location
        with open(location, "w") as outfile:
            outfile.write(ujson.dumps(self.serialize()))

    @property
    def pdf_location(self):
        self.load_data()
        return os.path.join(
            DEBATES_DIR, "%s_%s_%s.pdf" % ("debate", self.chamber, self.date)
        )

    @property
    def json_location(self):
        if self._json_location:
            return self._json_location

        return os.path.join(
            DEBATES_DIR, "%s_%s_%s.json" % ("debate", self.chamber, self.date)
        )

    @property
    def content_by_speaker(self):
        self.load_data()
        speakers = defaultdict(list)
        for section in self.debate_sections:
            for speech in section.speeches:
                if speech.member_obj is not None:
                    speakers[speech.member_obj.pid].extend(speech.paras)
                else:
                    # Couldn't get pid but want to include anyway
                    speakers[speech.by].extend(speech.paras)
        return speakers

    @property
    def timestamp(self):
        self.load_data()
        return datetime.datetime.strptime(self.date, "%Y-%m-%d")



import argparse
import datetime
import os
from multiprocessing.pool import ThreadPool

import ujson
import tqdm

from urllib.request import Request
from urllib.request import urlopen

from oireachtas_data.models.debate import Debate
from oireachtas_data.constants import DEBATES_DIR
from oireachtas_data.utils import get_debates


def scrape_debates(d):
    try:
        d.load_data()
        d.write()
    except:
        print("Possibly wrong pdf parsed: %s - %s" % (d.date, d.chamber))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--processes", type=int, dest="num_processes", default=20)
    args = parser.parse_args()

    if not os.path.exists(DEBATES_DIR):
        os.makedirs(DEBATES_DIR, exist_ok=True)

    earliest_possible = datetime.datetime(1900, 1, 1)
    chamber_types = ["house", "committee"]
    try:
        earliest = min([d.timestamp for d in get_debates()])
    except:
        earliest = datetime.datetime.now()

    # There are a lot of access denieds, should look here for missing data
    # https://www.oireachtas.ie/en/debates/debate/seanad/{x}/{y}/ from https://data.oireachtas.ie/akn/ie/debateRecord/seanad/{x}/debate/mul@/dbsect_{y}.xml
    # will need to parse from the page which will be anoying

    for chamber_type in chamber_types:
        while earliest > earliest_possible:
            earliest = earliest.replace(day=1)

            chamber_id = ""
            chamber = ""
            date_start = earliest.strftime("%Y-%m-%d")
            date_end = (
                (earliest + datetime.timedelta(days=32)).replace(day=1)
                - datetime.timedelta(days=1)
            ).strftime("%Y-%m-%d")
            limit = 1000

            print(
                "Get debates of config: %s"
                % (
                    {
                        "chamber_id": chamber_id,
                        "chamber": chamber,
                        "chamber_type": chamber_type,
                        "date_start": date_start,
                        "date_end": date_end,
                        "limit": limit,
                    }
                )
            )

            req = Request(
                "https://api.oireachtas.ie/v1/debates?chamber_type=%s&chamber_id=%s&chamber=%s&date_start=%s&date_end=%s&limit=%s"
                % (chamber_type, chamber_id, chamber, date_start, date_end, limit),
                headers={"accept": "application/json"},
            )
            response = urlopen(req)
            data = ujson.loads(response.read())

            print("got debates: %s" % (len(data["results"])))

            debates = []
            for d in data["results"]:
                debates.append(
                    Debate(
                        date=d["contextDate"],
                        chamber=d["debateRecord"]["chamber"]["showAs"],
                        counts=d["debateRecord"]["counts"],
                        debate_type=d["debateRecord"]["debateType"],
                        debate_sections=d["debateRecord"]["debateSections"],
                    )
                )

            pool = ThreadPool(processes=args.num_processes)
            for _ in tqdm.tqdm(
                pool.imap_unordered(scrape_debates, debates), total=len(debates)
            ):
                pass

            earliest = earliest - datetime.timedelta(days=1)


if __name__ == "__main__":
    main()



from oireachtas_data.utils import get_debates


def main():
    debates = get_debates()

    print("Loaded %s debates" % len(debates))
    print("Do something with the data in the variable 'debates'...")

    import pdb

    pdb.set_trace()

    pass


if __name__ == "__main__":
    main()



import os

from oireachtas_data.models.new_pdf import PDF as NEW_PDF
from oireachtas_data.models.old_pdf import PDF as OLD_PDF


def to_text(fp):
    text_file = fp + ".txt"

    if not os.path.exists(text_file):
        with open(fp, "rb") as f:
            os.system(f"pdftotext '{fp}' '{text_file}'")

    if not os.path.exists(text_file):
        print("Could not convert pdf to text: %s" % (text_file))
        return

    f = open(text_file, "r")
    data = f.read()
    f.close()

    return data


def pdf_parser_get(pdf_location):
    content = to_text(pdf_location)

    if content is None:
        print("Could not parse pdf type: %s" % (pdf_location))
        return

    if "��������������" in content:
        return NEW_PDF
    return OLD_PDF



import nltk

from oireachtas_data.models.member import Members

members = Members()
members.load()


try:
    nltk.data.find("taggers/averaged_perceptron_tagger")
except LookupError:
    nltk.download("averaged_perceptron_tagger")

try:
    nltk.data.find("tokenizers/punkt")
except LookupError:
    nltk.download("punkt")



import hashlib

from cached_property import cached_property

import nltk
from nltk.tag.perceptron import PerceptronTagger

TAGGER = PerceptronTagger()


class Paras:
    def __init__(self, data=None):
        self.data = data


class Para:
    """
    Paragraph object, has a title, id, and content.

    Inherits from Text which has all the text analysis bits
    """

    __slots__ = ("title", "eid", "content")

    def __init__(self, *args, **kwargs):
        """

        :kwarg title: Sometimes a paragraph has a title, not often
        :kwarg eid: Incremented id of the paragraph
        :kwarg content: The str text content
        """
        self.title = kwargs.get("title", None)
        self.eid = kwargs.get("eid", None)
        self.content = kwargs.get("content", None)

    @property
    def __dict__(self):
        return self.serialize()

    @staticmethod
    def parse(data):
        return Para(title=data["title"], eid=data["eid"], content=data["content"])

    @cached_property
    def words(self):
        """

        :return: A list of nltk tokenized words
        """
        return nltk.word_tokenize(self.content)

    @cached_property
    def tokens(self):
        """

        :reuturn: A list of nltk tokenized words with their tag
        """
        return TAGGER.tag(self.words)

    @property
    def content_hash(self):
        return int(hashlib.md5(bytes(self.content, "utf8")).hexdigest(), 16)

    def serialize(self):
        return {"title": self.title, "eid": self.eid, "content": self.content}
